@@ -5,7 +5,7 @@ use warnings;
 
 use parent qw(Tickit::ContainerWidget);
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Tickit::Widget::LogAny - log message rendering
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -51,11 +51,14 @@ Provides basic log rendering, with optional warn() / STDERR capture.
 
 <p>Basic rendering:</p>
 <p><img src="http://tickit.perlsite.co.uk/cpan-screenshot/tickit-widget-logany-basic.png" alt="Log::Any output displayed in Tickit widget" width="663" height="208"></p>
+<p>Stack trace popup:</p>
+<p><img src="http://tickit.perlsite.co.uk/cpan-screenshot/tickit-widget-logany-basic.png" alt="Log message with stack trace popup display using Tickit desktop layout" width="675" height="362"></p>
 
 =end HTML
 
 Activating any line in the list of log messages (typically by pressing C<Enter>) will
-show the stack trace for that entry.
+show the stack trace for that entry. Use the OK button to close (typically by pressing
+C<Tab>, then C<Enter>).
 
 =cut
 
@@ -68,6 +71,7 @@ use Variable::Disposition qw(dispose retain retain_future);
 use POSIX qw(strftime);
 use Text::Wrap (); 
 
+use Tickit::Utils qw(textwidth substrwidth);
 use Tickit::Style;
 use Tickit::Widget::Table;
 use Tickit::Widget::VBox;
@@ -94,6 +98,8 @@ BEGIN {
 		severity_info_fg      => 'green',
 		severity_debug_fg     => 'grey',
 		severity_trace_fg     => 'grey',
+		subname_fg            => 'hi-blue',
+		subname_b             => 1,
 		;
 }
 
@@ -290,12 +296,28 @@ sub show_stacktrace {
 		$holder->add(Tickit::Widget::Static->new(text => $_)) for @text;
 	}
 
-	my $tbl = Tickit::Widget::Table->new(
+	my $tbl;
+	$tbl = Tickit::Widget::Table->new(
 		columns => [
-			{ label => 'File' },
-			{ label => 'Line', width => 5 },
+			{ label => 'Location', transform => sub {
+				my ($row, $col, $cell) = @_;
+				$cell =~ s{^\Q$_/}{} for @INC;
+				Future->done($cell)
+			} },
 			{ label => 'Context', width => 8 },
-			{ label => 'Sub' },
+			{ label => 'Sub', transform => sub {
+				my ($row, $col, $cell) = @_;
+				my $w = $tbl->column_width($col) - textwidth($cell);
+				$cell = '...' . substrwidth($cell, 3-$w) if $w < 0;
+				my $pos = rindex($cell, '::') or return Future->done($cell);
+				Future->done(
+					String::Tagged->new(
+						$cell
+					)->apply_tag(
+						$pos + 2, -1, $self->get_style_pen('subname')->getattrs
+					)
+				)
+			} },
 		],
 		failure_transformations => [
 			sub { '' }
@@ -303,7 +325,10 @@ sub show_stacktrace {
 		item_transformations => [
 			sub {
 				my ($idx, $item) = @_;
-				Future->done([ map $_ // '', @{$item}{qw(filename line ctx sub)} ])
+				Future->done([
+					$item->{filename} . ':' . $item->{line},
+					@{$item}{qw(ctx sub)}
+				])
 			}
 		]
 	);
@@ -312,6 +337,8 @@ sub show_stacktrace {
 	my $win = $self->window;
 	$holder->add(
 		Tickit::Widget::Button->new(
+			class => 'stacktrace_ok',
+			style => { linetype => 'none' },
 			label => 'OK',
 			on_click => sub {
 				eval {
@@ -336,7 +363,7 @@ sub stacktrace_holder_widget {
 	$container = $self unless $container;
 
 	my $cleanup = sub { ... };
-	retain(my $vbox = Tickit::Widget::VBox->new);
+	retain(my $vbox = Tickit::Widget::VBox->new(style => { spacing => 1 }));
 	my $win = $self->window;
 	if($container->isa('Tickit::Widget::FloatBox')) {
 		$container->add_float(
